;*****************************************************
; Project : Home Wath
; Version : 1.3
; Date    : 08.01.2009
; Author  :   
; Comments: -  
;             (/)     ( )
;              
; Chip type           : ATTINY2313
; Clock frequency     : 8,0 MHz
;
; FUSE BITS - 0 - programmed/ 1- unprogrammed
; Calibrated Internal RC Oscillator 8MHz
; CKSEL3..0   - 0100
; Slowly rising power
; SUT1..0      - 10
; Divide clock by 8
; CKDIV8       - 1
; Output Clock on CKOUT pin
; CKOUT        - 1
; External Reset disable
; RSTDISBL     - 1
; Brown-out Detector trigger level
; BODLEVEL2..0 - 100 - reset     4,3
; Watchdog Timer always on
; WDTON        - 1
; Enable Serial Program and Data
; SPIEN        - 0
; EEPROM memory is preserved through the Chip Erase
; EESAVE       - 1
; DebugWIRE Enable
; DWEN         - 1
; Self Programming Enable (  SPMEN)
; SELFPRGEN    - 1
; 
;   :
; CKSEL3, CKSEL1, CKSEL0, SUT0, BODLEVEL1, BODLEVEL0
;*****************************************************

;         
;       
.equ Wait_Press_Escape      = 10

;    (/) ()
.equ DELAY_SHOW_SECONDS      = 4

;          
;        (1=0.0625)
; 4*0,0625 = 0,25
.equ Min_Value_TH_Up_Down   = 4

;     ()
.equ DELAY_READ_TH          = 60

;         
.equ DELAY_SHOW_TH          = 30

;*********************************************************** 
.include "tn2313def.inc"

;    100*4 
.equ CONST_TIMER0 = 256-8000000/256/(4*100)

;    *10
.equ DELAY_BLINK             = 50

;    WDT  *2,5  = 100ms
.equ DELAY_WDR               = 40

;  
.equ Button_Enter  = $08
.equ Button_Up     = $04
.equ Button_Down   = $02
.equ Button_Escape = $80

;    
.equ Delay_Press            = 06
;    +/-
.equ Delay_Repeat_First     = 50
;   +/-
.equ Delay_Repeat           = 10
;     Enter
.equ Delay_Repeat_Enter     = 200

.equ __i2c_port = PORTA
.equ __sda_bit  = 0
.equ __scl_bit  = 1

; BT
.equ __BT_pin   = PIND
.equ __BT_bit   = 0

.equ __I1W_port = PORTD
.equ __I1W_bit  = 1

.equ __DISP_port = PORTD
.equ __I1_bit   = 3
.equ __I2_bit   = 4
.equ __I3_bit   = 5
.equ __I4_bit   = 6

;***********************************************************
;    /  
.def Code_Display           = R2
;    
.def Addr_Display           = R3
;  
.def mSeconds               = R4
;  
.def Seconds                = R5
;  ms 
.def Wait_mSeconds          = R6
;    
.def mSecondsBlink          = R7
;        
.def WaitSeconds            = R8
;       
.def TimerPress             = R9
;         
.def TimerRepeat            = R10
;    
.def CodeKey                = R11
;   WDT
.def TimerWDT               = R12
;   
.def ModeTH                 = R13
;    
.def TimerSaveTH            = R14
;    
.def WaitShow               = R15
;***********************************************************
;  
.def RegFlags1              = R16
;  WDT
.equ F_WDT                  = 0
;  
.equ F_Blink                = 1
;   
.equ F_Repeat_Press_Key     = 2
;  
.equ F_Show                 = 3
;   
.equ F_Read_TH              = 4
;   
;      
;        
.equ F_Enable_Show_TH       = 5
;  
.equ F_TH_Down              = 6
;  
.equ F_TH_Up                = 7
;***********************************************************
.def RegFlags2              = R17
;   
.equ F_SET_WATH             = 0
;   
.equ F_SET_HOURS            = 1
;  RTC
.equ F_WRITE_WATH           = 2
;   
.equ F_SHOW_WATH            = 3
;     0 - , 1 - 
.equ F_TH_POINT             = 6
;   0 -   , 1 -   
.equ F_TYPE_DISP            = 7
;***********************************************************
;  
.def BufKey                 = R25
;***********************************************************
; SRAM $60-$DF -  
;    
.equ CSeconds               = $60
.equ CMinutes               = $61
.equ CHours                 = $62
;      
.equ SMinutes               = $63
.equ SHours                 = $64
;***********************************************************
;    Display
.equ Display1               = $70
.equ Display2               = $71
.equ Display3               = $72
.equ Display4               = $73
;    BlinkDisplay
.equ BlinkDisplay1          = $74
.equ BlinkDisplay2          = $75
.equ BlinkDisplay3          = $76
.equ BlinkDisplay4          = $77
;***********************************************************
; 
.equ LSB     = $80
.equ MSB     = $81
.equ UTH     = $82
.equ UTL     = $83
.equ Config  = $84
.equ RU1     = $85
.equ RU2     = $86
.equ RU3     = $87
.equ CRC     = $88
;***************************************************************************
;    
.equ Code_TH = $8A
;.equ C1_TH   = $8B
;.equ C2_TH   = $8C
;.equ C3_TH   = $8D
;.equ C4_TH   = $8E
;
;***************************************************************************
;     
.equ TL0   = $A0
;.equ TH0   = $A1
;.equ TL1   = $A2
;.equ TH1   = $A3
;.equ TL2   = $A4
;.equ TH2   = $A5
;.equ TL3   = $A6
;.equ TH3   = $A7
;.equ TL4   = $A8
;.equ TH4   = $A9

;***************************************************************************
;  
;***************************************************************************
;0000 Reset
       RJMP  Reset
;0001  INT0
       RJMP  IntIRQ0
;0002 INT1
       RETI
;0003 TIMER1 CAPT (Capture Event)
       RETI
;0004 TIMER1 COMPA (Compare Match A)
       RETI
;0005 TIMER1 OVF (Overflow)
       RETI
;0006 TIMER0 OVF (Overflow)
       RJMP  IntTimer0
;0007 USART0 RX (RX Complete)
       RETI
;0008 USART0 UDRE (Data Register Empty)
       RETI
;0009 UART TX (TX Complete)
       RETI
;000A ANALOG COMP (Comparator)
       RETI
;000B PCINT (Pin Change Interrupt)
       RETI
;000C TIMER1 COMPB (Compare Match B)
       RETI
;000D TIMER0 COMPA (Compare Match A)
       RETI
;000E TIMER0 COMPB (Compare Match B)
       RETI
;000F USI START (USI Start Condition)
       RETI
;0010 USI OVERFLOW (USI Overflow)
       RETI
;0011 EE READY (EEPROM Ready)
       RETI
;0012 WDT OVERFLOW (Watchdog Timer Overflow)
       RETI
;***********************************************************
;   01
; 20  02
;   40
; 10  04
;   08  80
;*********************************************************** 
T_Number:
;            0    1    2    3    4    5    6    7
       .db $3F, $06, $5B, $4F, $66, $6D, $7D, $07
;            8    9    A    B    C    d   E    F
       .db $7F, $6F ;, $77, $7C, $39, $5E, $79, $71

;**********************************************************
; . .:  R30 - 
; . .: R0 -  
;**********************************************************
LPM_Number:
       ANDI  R30,$F
       MOV   R0,R30
       LDI   ZH,HIGH(T_Number<<1)
       LDI   ZL,LOW( T_Number<<1)
       ADD   ZL,R0
       CLR   R0
       ADC   ZH,R0
       LPM
       RET
;**********************************************************
;    
;**********************************************************
IntTimer0:
       PUSH  R30
       IN    R30,SREG
       PUSH  R30
       PUSH  R31
       PUSH  YL
;      5 
       LDI   R30,CONST_TIMER0
       OUT   TCNT0,R30
;**********************************************************
;   Wathdog
       MOV   R30,TimerWDT
       INC   R30
       CPI   R30,DELAY_WDR
       BRCS  IntTimer0_Set_TimerWDT
       CLR   R30
;    WDT
       SBR   RegFlags1,1<<F_WDT
IntTimer0_Set_TimerWDT:
       MOV   TimerWDT,R30
;**********************************************************
;    
       MOV   R31,Addr_Display
       INC   R31
       MOV   R30,Code_Display
       LSR   R30
       BRNE  IntTimer0_Set_Code_Display
       LDI   R30,$08
       CLR   R31
IntTimer0_Set_Code_Display:
       MOV   Code_Display,R30
       MOV   Addr_Display,R31
;  Display  
       LDI   YL,Display1
       ADD   YL,Addr_Display
;    
       LD    R30,Y
       SBRC  RegFlags1,F_BLINK
       LDD   R30,Y+4
       PUSH  R30
;  
       LDI   R31,(1<<__I1_bit)|(1<<__I2_bit)|(1<<__I3_bit)|(1<<__I4_bit)
       SBRS  RegFlags2,F_TYPE_DISP
       COM   R31
       ANDI  R31,(1<<__I1_bit)|(1<<__I2_bit)|(1<<__I3_bit)|(1<<__I4_bit)
       OUT   PORTD,R31
       CLR   R31
       SBRS  RegFlags2,F_TYPE_DISP
       COM   R31
       OUT   PORTB,R31
; ~Addr_Code_Display
       MOV   R30,Code_Display
       ANDI  R30,$0F
       SWAP  R30
       LSR   R30
;  
       SBRC  RegFlags2,F_TYPE_DISP
       COM   R30
       ANDI  R30,(1<<__I1_bit)|(1<<__I2_bit)|(1<<__I3_bit)|(1<<__I4_bit)
       OUT   PORTD,R30
;  
       POP   R30
       SBRS  RegFlags2,F_TYPE_DISP
       COM   R30
       OUT   PORTB,R30
;  
       RCALL Key
;   
       TST   Addr_Display
       BRNE  IntTimer0_End
;**********************************************************
;   
       MOV   R30,mSecondsBlink
       INC   R30
       CPI   R30,DELAY_BLINK
       BRCS  IntTimer0_Set_mSeconds_Blink
;   
       LDI   R30,1<<F_BLINK
       EOR   RegFlags1,R30
       CLR   R30
IntTimer0_Set_mSeconds_Blink:
       MOV   mSecondsBlink,R30
;**********************************************************
;   
       TST   Wait_mSeconds
       BREQ  IntTimer0_Wait_mSeconds_Ok
       DEC   Wait_mSeconds
IntTimer0_Wait_mSeconds_Ok:
;**********************************************************
;  
       MOV   R30,mSeconds
       INC   R30
       CPI   R30,100
       BRCS  IntTimer0_Set_mSeconds
       CLR   R30
IntTimer0_Set_mSeconds:
       MOV   mSeconds,R30
       TST   R30
       BRNE  IntTimer0_End
;**********************************************************
;  
       INC   Seconds
       LDI   R30,DELAY_READ_TH
       CP    Seconds,R30
       BRCS  IntTimer0_Seconds_Ok
       CLR   Seconds
;    
       SBR   RegFlags1,1<<F_Read_TH
IntTimer0_Seconds_Ok:
;**********************************************************
;     
       SBRS  RegFlags2,F_SET_WATH
       RJMP  IntTimer0_WaitSeconds_Ok
       INC   WaitSeconds
       LDI   R30,Wait_Press_Escape
       CP    WaitSeconds,R30
       BRCS  IntTimer0_WaitSeconds_Ok
       CLR   WaitSeconds
       LDI   BufKey,Button_Escape
IntTimer0_WaitSeconds_Ok:
;**********************************************************
;     
       INC   WaitShow
       LDI   R30,DELAY_SHOW_SECONDS
       CP    WaitShow,R30
       BRCS  IntTimer0_WaitShow_Ok
       CLR   WaitShow
;    
       LDI   R30,1<<F_SHOW_WATH
       EOR   RegFlags2,R30
;    
       SBR   RegFlags1,1<<F_SHOW
IntTimer0_WaitShow_Ok:
;**********************************************************
;     
       SBRS  RegFlags1,F_Enable_Show_TH
       RJMP  IntTimer0_TimerSaveTH_Ok
       SBRS  RegFlags1,F_Read_TH
       RJMP  IntTimer0_TimerSaveTH_Ok
       INC   TimerSaveTH
       LDI   R30,DELAY_SHOW_TH
       CP    TimerSaveTH,R30
       BRCS  IntTimer0_TimerSaveTH_Ok
       CLR   TimerSaveTH
;     
       CBR   RegFlags1,1<<F_Enable_Show_TH
IntTimer0_TimerSaveTH_Ok:
IntTimer0_End:
       POP   YL
       POP   R31
       POP   R30
       OUT   SREG,R30
       POP   R30
       RETI
;***********************************************************
;  
;***********************************************************
Key:
;   
       IN    R30,__BT_pin
       SBRC  RegFlags2,F_TYPE_DISP
       COM   R30
       SBRS  R30,__BT_bit
       RJMP  Key_Up
       CLR   WaitSeconds
       MOV   R30,CodeKey
       AND   R30,Code_Display
       BRNE  Key_Timer
;    
       OR    CodeKey,Code_Display
       LDI   R30,Delay_Press
       MOV   TimerPress,R30
       LDI   R30,Delay_Repeat_First
       MOV   TimerRepeat,R30
;      
       CBR   RegFlags1,1<<F_Repeat_Press_Key
       RJMP  Key_Repeat
Key_Up:
       MOV   R30,CodeKey
       AND   R30,Code_Display
       BREQ  Key_Exit
       TST   TimerPress
       BRNE  Key_Exit_No_Press
       SBRS  RegFlags1,F_Repeat_Press_Key
       MOV   BufKey,CodeKey
Key_Exit_No_Press:
       CLR   CodeKey
Key_Exit:
       RET
Key_Timer:
;   (   )
       TST   TimerPress
       BREQ  Key_Press_Ok
       DEC   TimerPress
       RET
Key_Press_Ok:
       TST   TimerRepeat
       BREQ  Key_Ok
       DEC   TimerRepeat
       RET
Key_Ok:
       MOV   BufKey,CodeKey
;      
       SBR   RegFlags1,1<<F_Repeat_Press_Key
       LDI   R30,Delay_Repeat
       MOV   TimerRepeat,R30
Key_Repeat:
;     Enter
       LDI   R30,Button_Enter
       CP    CodeKey,R30
       BRNE  Key_Repeat_End
       LDI   R30,Delay_Repeat_Enter
       MOV   TimerRepeat,R30
Key_Repeat_End:
       RET
;**********************************************************
;  
;**********************************************************
Fill_Display:
       LDI   YL,Display1
       LDI   R31,8
Fill_Display_Do:
       ST    Y+,R30
       DEC   R31
       BRNE  Fill_Display_Do
       RET
;**********************************************************
; 
;**********************************************************
Reset:
;  
       CLI
       CLR   R30
;   R0-R30
       CLR  YH
       LDI  YL,28
Reset_Clear_Reg:
       ST   -Y,R30
       TST  YL
       BRNE Reset_Clear_Reg
;  
       LDI   YL,$60
Reset_Clr_RAM:
       ST    Y+,R30
       CPI   YL,Low(RAMEND+1)
       BRNE  Reset_Clr_RAM
; 
       LDI   R31,Low(RAMEND)
       OUT   SPL,R31
;
       CLR   R30
; Port A initialization
       OUT   PORTA,R30
       OUT   DDRA,R30
;   
       SBIC  __BT_pin,__BT_bit
       SBR   RegFlags2,1<<F_TYPE_DISP
; Port B initialization
       LDI   R31,$FF
       SBRC  RegFlags2,F_TYPE_DISP
       CLR   R31
       OUT   PORTB,R31
       LDI   R31,$FF
       OUT   DDRB,R31
; Port D initialization
       OUT   PORTD,R30
       LDI   R31,(1<<__I1_bit)|(1<<__I2_bit)|(1<<__I3_bit)|(1<<__I4_bit)
       OUT   DDRD,R31
; Timer/Counter 0 initialization
; Clock source: System Clock
; Clock value: 28,800 kHz (System Clock/256)
       LDI   R31,(1<<CS02)+(0<<CS01)+(0<<CS00)
       OUT   TCCR0,R31
       LDI   R31,CONST_TIMER0
       OUT   TCNT0,R31
; Timer/Counter 1 initialization
; Clock source: System Clock
; Clock value: Timer 1 Stopped
; Mode: Normal top=FFFFh
; OC1 output: Discon.
; Noise Canceler: Off
; Input Capture on Falling Edge
       OUT   TCCR1A,R30
       OUT   TCCR1B,R30
       OUT   TCNT1H,R30
       OUT   TCNT1L,R30
       OUT   OCR1AH,R30
       OUT   OCR1AL,R30
; External Interrupt(s) initialization
; INT0: On
; INT0 Mode: Falling Edge
; INT1: Off
       LDI   R31,(0<<INT1) | (1<<INT0)
       OUT   GIMSK,R31
       LDI   R31,(0<<SE) | (0<<SM) | (0<<ISC11) | (0<<ISC10) | (1<<ISC01) | (0<<ISC00)
       OUT   MCUCR,R31
       LDI   R31,(0<<INTF1) | (1<<INTF0)
       OUT   GIFR,R31
; Timer(s)/Counter(s) Interrupt(s) initialization
       LDI   R31,(1<<TOIE0)
       OUT   TIMSK,R31
; UART initialization
; Communication Parameters: 8 Data, 1 Stop, No Parity
; UART Receiver: Off
; UART Transmitter: Off
       OUT   UCR,R30
       OUT   UBRR,R30
; Analog Comparator initialization
; Analog Comparator: Off
; Analog Comparator Input Capture by Timer/Counter 1: Off
       LDI   R31,(1<<ACD)
       OUT   ACSR,R31
; Watchdog Timer initialization
; Watchdog Timer Prescaler: OSC/16k = 0,125 s
       WDR
       IN    R31,WDTCSR
       ORI   R31,(1<<WDE) | (1<<WDCE)
       OUT   WDTCSR,R31
       LDI   R31,(1<<WDE) | (1<<WDCE) | (0<WDP3) | (0<WDP2) | (1<<WDP1) | (1<<WDP0)
       OUT   WDTCSR,R31
;  
       RCALL Write_RTC_Control
;  
       RCALL Read_RTC
;   
;    
       SER   R30
       RCALL Fill_Display
;  
       SEI
Main_Test_Display:
;  Wathdog
       SBRC  RegFlags1,F_WDT
       RCALL __WDR
       TST   Seconds
       BREQ  Main_Test_Display
;  
       CLR   R30
       RCALL Fill_Display
;   
       SBR   RegFlags2,1<<F_SHOW_WATH
;        T
       SBR   RegFlags1,(1<<F_SHOW)| (1<<F_Read_TH)
;**********************************************************
;   
;**********************************************************
Main_Loop:
;  Wathdog
       SBRC  RegFlags1,F_WDT
       RCALL __WDR
;  
       RCALL Key_Work
;     
       SBRC  RegFlags1,F_SHOW
       RCALL Show_Parameters
;  
       SBRC  RegFlags1,F_Read_TH
       RCALL Mode_TH
       RJMP  Main_Loop
;***************************************************************************
;  WDT
;***************************************************************************
__WDR:
       CBR   RegFlags1,1<<F_WDT
       WDR
       RET
;***********************************************************
;   
;***********************************************************
Show_Parameters:
;  
       CBR   RegFlags1,1<<F_SHOW
       LDI   YL,Display1
;   
       SBRC  RegFlags2,F_SHOW_WATH
       RJMP  Show_Time
;     
       SBRC  RegFlags1,F_Enable_Show_TH
       RJMP  Show_TH
;***************************************************************************
;  
;***************************************************************************
Show_Time:
       LDI   ZL,CMinutes
       CLI
       LD    R24,Z
       LDD   R23,Z+1
       SEI
;
       MOV   R30,R23
       RCALL BIN2BCD
       PUSH  R30
       CLR   R0
       CPI   R30,10
       BRCS  Show_Time_Skip_DH
; 
       SWAP  R30
       RCALL LPM_Number
Show_Time_Skip_DH:
       MOV   R18,R0
       POP   R30
       RCALL LPM_Number
; 
       MOV   R19,R0
; 
       MOV   R30,R24
       RCALL BIN2BCD
       PUSH  R30
       SWAP  R30
       RCALL LPM_Number
       MOV   R20,R0
       POP   R30
       RCALL LPM_Number
       MOV   R21,R0
;   
       SBRC  RegFlags2,F_SET_WATH
       RJMP  Show_Setup_Time
;  
       STD   Y+4,R18
       ST    Y,R18
;  
       STD   Y+5,R19
       SBR   R19,1<<7
       STD   Y+1,R19
;  
       STD   Y+6,R20
       STD   Y+2,R20
;  
       STD   Y+7,R21
       STD   Y+3,R21
       RET
;***************************************************************************
;  
;***************************************************************************
Show_TH:
;    
       SBRC  RegFlags2,F_SET_WATH
       RJMP  Show_Setup_Time
;   
       LDI   XL,Code_TH
       LD    R30,X+
       ST    Y,R30
       STD   Y+4,R30
       LD    R30,X+
       STD   Y+1,R30
       STD   Y+5,R30
       LD    R30,X+
       STD   Y+2,R30
       STD   Y+6,R30
       LD    R30,X+
       STD   Y+3,R30
       SBRC  RegFlags1,F_TH_Up
       LDI   R30,$0C
       SBRC  RegFlags1,F_TH_Down
       LDI   R30,$03
       STD   Y+7,R30
       RET
;***************************************************************************
;   
;***************************************************************************
Show_Setup_Time:
       LDI   XL,SMinutes
       LD    R24,X+
       LD    R23,X+
;
       MOV   R30,R23
       RCALL BIN2BCD
       PUSH  R30
       CLR   R0
       CPI   R30,10
       BRCS  Show_Setup_Time_Skip_DH
; 
       SWAP  R30
       RCALL LPM_Number
Show_Setup_Time_Skip_DH:
       MOV   R18,R0
       ST    Y,R0
       POP   R30
       RCALL LPM_Number
; 
       MOV   R19,R0
       MOV   R30,R0
       SBR   R30,1<<7
       STD   Y+1,R30
; 
       MOV   R30,R24
       RCALL BIN2BCD
       PUSH  R30
       SWAP  R30
       RCALL LPM_Number
       MOV   R20,R0
       STD   Y+2,R0
       POP   R30
       RCALL LPM_Number
       MOV   R21,R0
       STD   Y+3,R0
;    
       SBRS  RegFlags2,F_SET_HOURS
       RJMP  Show_Setup_Time_Minits_Set
;   
       CLR   R30
       STD   Y+4,R30
       LDI   R30,$80
       STD   Y+5,R30
       STD   Y+6,R20
       STD   Y+7,R21
       RET
Show_Setup_Time_Minits_Set:
;   
       STD   Y+4,R18
       STD   Y+5,R19
       CLR   R30
       STD   Y+6,R30
       STD   Y+7,R30
       RET
;-----------------------------------------------------------
; I2C
; 
; 5
; F SCL - 100
;-----------------------------------------------------------

;***********************************************************
; F() * 1,67
.equ Delay_QP = 13
; F() * 1,67 * 2
.equ Delay_HP = 27

.equ __i2c_dir=__i2c_port-1
.equ __i2c_pin=__i2c_port-2
;**********************************************************
;I2C_Init:
;      CBI  __i2c_port,__scl_bit
;      CBI  __i2c_port,__sda_bit
;      SBI  __i2c_dir,__scl_bit
;      CBI  __i2c_dir,__sda_bit
;      RJMP I2C_HP_Delay
;**********************************************************
;  C - 
;**********************************************************
I2C_Start:
      CBI   __i2c_dir,__sda_bit
      CBI   __i2c_dir,__scl_bit
      SEC
      NOP
      SBIS  __i2c_pin,__sda_bit
      RET
      SBIS  __i2c_pin,__scl_bit
      RET
      RCALL I2C_QP_Delay
      SBI   __i2c_dir,__sda_bit
      RCALL I2C_QP_Delay
      SBI   __i2c_dir,__scl_bit
      CLC
;***********************************************************
I2C_QP_Delay:
      PUSH  R30
      LDI   R30,Delay_QP
      RJMP  I2C_Delay_Go
;**********************************************************
;**********************************************************
I2C_Stop:
      SBI   __i2c_dir,__sda_bit
      SBI   __i2c_dir,__scl_bit
      RCALL I2C_HP_Delay
      CBI   __i2c_dir,__scl_bit
      RCALL I2C_QP_Delay
      CBI   __i2c_dir,__sda_bit
;**********************************************************
I2C_HP_Delay:
      PUSH  R30
      LDI   R30,Delay_HP
I2C_Delay_Go:
      DEC   R30
      BRNE  I2C_Delay_Go
      POP   R30
      RET
;**********************************************************
; . .: T Flag - ASC
; . .: R30 - Data
;**********************************************************
I2C_Read:
      LDI   R30,1
I2C_Read_Bit:
      CBI   __i2c_dir,__scl_bit
      RCALL I2C_HP_Delay
      CLC
      SBIC  __i2c_pin,__sda_bit
      SEC
      SBI   __i2c_dir,__scl_bit
      RCALL I2C_HP_Delay
      ROL   R30
      BRCC  I2C_Read_Bit
      BRTC  I2C_Read_ASC
      CBI   __i2c_dir,__sda_bit
      RJMP  I2C_Read_Pulse_SCL
I2C_Read_ASC:
      SBI   __i2c_dir,__sda_bit
I2C_Read_Pulse_SCL:
      RCALL I2C_QP_Delay
      CBI  __i2c_dir,__scl_bit
      RCALL I2C_HP_Delay
      SBI   __i2c_dir,__scl_bit
      RCALL I2C_QP_Delay
      CBI   __i2c_dir,__sda_bit
      RJMP I2C_QP_Delay
;**********************************************************
; . .: P30 - Data
; . .: C Flag - Error
;**********************************************************
I2C_Write:
       SEC
       ROL   R30
       RJMP  I2C_Write_First
I2C_Write_Bit:
       LSL   R30
       BREQ  I2C_Write_ACK
I2C_Write_First:
       BRCC  I2C_Write_Low
       CBI   __i2c_dir,__sda_bit
       RJMP  I2C_Write_Pulse_SCL
I2C_Write_Low:
       SBI   __i2c_dir,__sda_bit
I2C_Write_Pulse_SCL:
       RCALL I2C_HP_Delay
       CBI   __i2c_dir,__scl_bit
       RCALL I2C_HP_Delay
       SBI   __i2c_dir,__scl_bit
       RJMP  I2C_Write_Bit
I2C_Write_ACK:
       CBI   __i2c_dir,__sda_bit
       RCALL I2C_QP_Delay
       CBI   __i2c_dir,__scl_bit
       RCALL I2C_HP_Delay
       SEC
       SBIC  __i2c_pin,__sda_bit
       CLC
       SBI   __i2c_dir,__scl_bit
       RET
;-----------------------------------------------------------
; DS1307
; DS1307 (08h-3Fh)
.equ DS1307W = $D0
.equ DS1307R = $D1
;-----------------------------------------------------------

;**********************************************************
;     
;       
;**********************************************************
IntIRQ0:
       PUSH  R30
       IN    R30,SREG
       PUSH  R30
       PUSH  YL
       LDI   YL,CSeconds
;  
       LD    R30,Y
       INC   R30
       CPI   R30,60
       BRCS  IntIRQ0_S
       CLR   R30
IntIRQ0_S:
       ST    Y,R30
       BRNE  IntIRQ0_End
;  
       LDD   R30,Y+1
       INC   R30
       CPI   R30,60
       BRCS  IntIRQ0_M
       CLR   R30
IntIRQ0_M:
       STD   Y+1,R30
       BRNE  IntIRQ0_End
;  
       LDD   R30,Y+2
       INC   R30
       CPI   R30,24
       BRCS  IntIRQ0_H
       CLR   R30
IntIRQ0_H:
       STD   Y+2,R30
IntIRQ0_End:
;    
       SBR   RegFlags1,1<<F_SHOW
       POP   YL
       POP   R30
       OUT   SREG,R30
       POP   R30
       RETI
;***********************************************************
;  
; CHours, SHours
; CMinutes, SMinutes
; CSeconds
;***********************************************************
Read_RTC:
       RCALL  I2C_Start
       LDI    R30,DS1307W
       RCALL  I2C_Write
       CLR    R30
       RCALL  I2C_Write
       RCALL  I2C_Start
       LDI    R30,DS1307R
       RCALL  I2C_Write
; 
       CLT
       RCALL  I2C_Read
       MOV    R18,R30
; 
       CLT
       RCALL  I2C_Read
       MOV    R19,R30
; 
       SET
       RCALL  I2C_Read
       MOV    R20,R30
       RCALL  I2C_Stop
;    BIN 
       MOV    R30,R20
       RCALL  BCD2BIN
       BRCS   Read_RTC_Error
       CPI    R30,24
       BRCC   Read_RTC_Error
       MOV    R20,R30
;   BIN 
       MOV    R30,R19
       RCALL  BCD2BIN
       BRCS   Read_RTC_Error
       CPI    R30,60
       BRCC   Read_RTC_Error
       MOV    R19,R30
;   BIN 
       MOV    R30,R18
       RCALL  BCD2BIN
       BRCS   Read_RTC_Error
       CPI    R30,60
       BRCC   Read_RTC_Error
       MOV    R18,R30
;  
       LDI    YL,CSeconds
       CLI
       STD    Y+0,R18
       STD    Y+1,R19
       STD    Y+2,R20
       SEI
       STD    Y+3,R19
       STD    Y+4,R20
       RET
Read_RTC_Error:
       CLR    R19
       CLR    R20
       RJMP   Write_RTC
;***********************************************************
;    (BIN )
; SHours   - 
; SMinutes - 
;***********************************************************
Write_RTC_Setup:
;  
       LDI   XL,SHours
       LD    R30,X
       RCALL BIN2BCD
       MOV   R20,R30
       LD    R30,-X
       RCALL BIN2BCD
       MOV   R19,R30
;***********************************************************
;    (BCD )
; R20 - 
; R19 - 
;***********************************************************
Write_RTC:
       RCALL  I2C_Start
       LDI    R30,DS1307W
       RCALL  I2C_Write
       CLR    R30
       RCALL  I2C_Write
; 
       CLR    R30
       RCALL  I2C_Write
; 
       MOV    R30,R19
       RCALL  I2C_Write
; 
       MOV    R30,R20
       RCALL  I2C_Write
;  , , ,  
       LDI    R31,4
Write_RTC_Do:
       LDI    R30,1
       RCALL  I2C_Write
       DEC    R31
       BRNE   Write_RTC_Do
Write_RTC_Control_Ok:
;      
       LDI    R30,$10
       RCALL  I2C_Write
       RJMP   I2C_Stop
;***********************************************************
;   1     
;***********************************************************
Write_RTC_Control:
       RCALL  I2C_Start
       LDI    R30,DS1307W
       RCALL  I2C_Write
       LDI    R30,$07
       RCALL  I2C_Write
       RJMP   Write_RTC_Control_Ok
;***********************************************************
; DS18B20
;
; Memory Map:
;
;   Scratchpad      Byte           E2RAM
; Temperature LSB     0
; Temperature MSB     1
; TH/User Byte 1      2         TH/User Byte 1
; TL/User Byte 2      3         TL/User Byte 2
; Config              4         Config
; Reserved            5
; Reserved            6
; Reserved            7
; CRC                 8
;
; ROM Function Commands:
; $33 - Read ROM
; $55 - Match ROM
; $F0 - Search ROM
; $EC - Alarm search
; $CC - Skip ROM
;
; Memory Function Commands:
; $4E - Write Scratchpad
; $BE - Read Scratchpad
; $48 - Copy Scratchpad
; $44 - Convert T
; $B8 - Recall E2
; $B4 - Read Power Supply
;
; Transaction sequence:
; Initialisation
; ROM Function Command
; Memory Function Command
; Transaction/Data

.equ DS18B20SkipROM       = $CC
.equ DS18B20ConvertT      = $44
.equ DS18B20ReadScrathpad = $BE

;    ( http://files.dalsemi.com/auto_id/public/an126.zip)
; T = 1/F = 1/8 
; 1 = 8instr
; k = 8/2 = 4
; n = t*k

.equ DELAY_6us   = 6*3
.equ DELAY_9us   = 9*3
.equ DELAY_10us  = 10*3
.equ DELAY_45us  = 45*3

.equ DELAY_70us  = 2*70
.equ DELAY_410us = 2*410
.equ DELAY_480us = 2*480

;***********************************************************
;***********************************************************
I1W_Delay_70us:
       LDI   ZH,High(DELAY_70us)
       LDI   ZL,Low( DELAY_70us)
       RJMP  I1W_Delay_W
;***********************************************************
;***********************************************************
I1W_Delay_410us:
       LDI   ZH,High(DELAY_410us)
       LDI   ZL,Low( DELAY_410us)
       RJMP  I1W_Delay_W
;***********************************************************
;***********************************************************
I1W_Delay_480us:
       LDI   ZH,High(DELAY_480us)
       LDI   ZL,Low( DELAY_480us)
I1W_Delay_W:
       SBIW  ZL,1
       BRNE  I1W_Delay_W
       RET
;***********************************************************
;***********************************************************
I1W_Reset:
;  
       CLI
       SBI   __I1W_port-1,__I1W_bit
       RCALL I1W_Delay_480us
       CBI   __I1W_port-1,__I1W_bit
       RCALL I1W_Delay_70us
       SBIC  __I1W_port-2,__I1W_bit
       RJMP  I1W_Reset_Error
       RCALL I1W_Delay_410us
       SBIS  __I1W_port-2,__I1W_bit
       RJMP  I1W_Reset_Error
       CLC
I1W_Reset_End:
;  
       SEI
       RET
I1W_Reset_Error:
       SEC
       RJMP  I1W_Reset_End
;***********************************************************
; ..  R30 -  
; .. R30 -  
;***********************************************************
ReadWrite1Wire:
;  
       CLI
       PUSH  R31
       LDI   R31,8
ReadWrite1Wire_NextBit:
       PUSH  R31
       SBI   __I1W_port-1,__I1W_bit
       LDI   R31,DELAY_6us
ReadWrite1Wire_NextBit_Delay_1:
       DEC   R31
       BRNE  ReadWrite1Wire_NextBit_Delay_1
       SBRC  R30,0
       CBI   __I1W_port-1,__I1W_bit
       LDI   R31,DELAY_9us
ReadWrite1Wire_NextBit_Delay_2:
       DEC   R31
       BRNE  ReadWrite1Wire_NextBit_Delay_2
       LSR   R30
       SBIC  __I1W_port-2,__I1W_bit
       SBR   R30,(1<<7)
       LDI   R31,DELAY_45us
ReadWrite1Wire_NextBit_Delay_3:
       DEC   R31
       BRNE  ReadWrite1Wire_NextBit_Delay_3
       CBI   __I1W_port-1,__I1W_bit
       LDI   R31,DELAY_10us
ReadWrite1Wire_NextBit_Delay_4:
       DEC   R31
       BRNE  ReadWrite1Wire_NextBit_Delay_4
       POP   R31
       DEC   R31
       BRNE  ReadWrite1Wire_NextBit
       POP   R31
;  
       SEI
       RET
;***********************************************************
; .. R30 -  
;         R31 -  CRC
;***********************************************************
Do_CRC:
       PUSH  R28
       PUSH  R29
       LDI   R28,8
Do_CRC_Loop:
       MOV   R29,R30
       EOR   R29,R31
       LSR   R29
       BRCC  Do_CRC_Zero
       LDI   R29,$18
       EOR   R31,R29
Do_CRC_Zero:
       ROR   R31
       CLC
       SBRC  R30,0
       SEC
       ROR   R30
       DEC   R28
       BRNE  Do_CRC_Loop
       POP   R29
       POP   R28
       RET
;***********************************************************
;    
;***********************************************************
ConvertThDS18B20:
       RCALL I1W_Reset
       BRCS  Mode_TH_Clr_ModeTH
       LDI   R30,DS18B20SkipROM
       RCALL ReadWrite1Wire
       LDI   R30,DS18B20ConvertT
       RCALL ReadWrite1Wire
;  
       RJMP  Mode_TH_Inc
;***********************************************************
;  
;***********************************************************
ReadThDS18B20:
       RCALL I1W_Reset
       BRCS  Mode_TH_Clr_ModeTH
       LDI   R30,DS18B20SkipROM
       RCALL ReadWrite1Wire
       LDI   R30,DS18B20ReadScrathpad
       RCALL ReadWrite1Wire
       LDI   XL,LSB
       LDI   R31,9
ReadThDS18B20_Go:
       LDI   R30,$FF
       RCALL ReadWrite1Wire
       ST    X+,R30
       DEC   R31
       BRNE  ReadThDS18B20_Go
; CRC
       CLR   R31
       LDI   XL,LSB
       LDI   R29,8
ReadThDS18B20_CRC:
       LD    R30,X+
       RCALL Do_CRC
       DEC   R29
       BRNE  ReadThDS18B20_CRC
       LD    R30,X
       EOR   R30,R31
       BRNE  Mode_TH_Clr_ModeTH
;  
       RJMP  Mode_TH_Inc
;***********************************************************
;  
;***********************************************************
Mode_TH:
       CLR   R30
       CP    ModeTH,R30
;  
       BREQ  ConvertThDS18B20
       INC   R30
       CP    ModeTH,R30
;  1 
       BREQ  Wait_TH
       INC   R30
       CP    ModeTH,R30
;  
       BREQ  ReadThDS18B20
       INC   R30
       CP    ModeTH,R30
       BREQ  Calc_TH
;   
       CBR   RegFlags1,(1<<F_Read_TH)
;     
       SBR   RegFlags1,1<<F_Enable_Show_TH
;    
       CLR   TimerSaveTH
Mode_TH_Clr_ModeTH:
;      
       CLR   ModeTH
       RET
Mode_TH_Inc:
       INC   ModeTH
       LDI   R30,100
       MOV   Wait_mSeconds,R30
       RET
;***********************************************************
;  1 
;***********************************************************
Wait_TH:
;  
       TST   Wait_mSeconds
       BRNE  Wait_TH_End
       RCALL Mode_TH_Inc
Wait_TH_End:
       RET
;***********************************************************
;  
;***********************************************************
Calc_TH:
;     
       RCALL Mem_Up_Down_TH
;   -  
       LDI   XL,LSB
       LD    R18,X+
       LD    R19,X
       RCALL CalcThDS18B20
       LDI   XL,Code_TH
       MOV   YL,XL
       LD    R18,X+
       LD    R19,X+
       LD    R20,X+
       LD    R21,X+
       LD    R22,X+
;  "-"
       LDI   R23,$40
;   ?
       SBRC  R18,6
       RJMP  Calc_TH_FD
;   ?
       SBRC  R18,7
       RJMP  Calc_TH_FM
;***********************************************************
;    ,  
;***********************************************************
       LDI   XL,Code_TH+2
       TST   R20
       BRNE  Calc_TH_FP_1
       ST    Y+,R20
       TST   R21
       BRNE  Calc_TH_FP_2
       ST    Y+,R21
       RJMP  Calc_TH_FP_3
Calc_TH_FP_1:
;1
       MOV   R30,R20
       RCALL SymToBufDisp
;2
Calc_TH_FP_2:
       MOV   R30,R21
       RCALL SymToBufDisp
;3
Calc_TH_FP_3:
       MOV   R30,R22
       RCALL SymToBufDisp
;4
Calc_TH_FP_4:
;   $39 - "C", $63 - "0"
       LDI   R30,$63
       ST    Y+,R30
;  
       RJMP  Mode_TH_Inc
;***********************************************************
;    ,  
;***********************************************************
Calc_TH_FM:
       LDI   XL,Code_TH+2
       TST   R21
       BRNE  Calc_TH_FM_1
       ST    Y+,R21
       ST    Y+,R23
       LDI   XL,Code_TH+2
       RJMP  Calc_TH_FP_3
Calc_TH_FM_1:
       ST    Y+,R23
       RJMP  Calc_TH_FP_2
;***********************************************************
;    ,   ,
;  
;***********************************************************
Calc_TH_FD:
;   ?
       SBRC  R18,7
       RJMP  Calc_TH_FDM
       TST   R19
       BRNE  Calc_TH_FD1
       TST   R20
       BRNE  Calc_TH_FD2
       ST    Y+,R20
       RJMP  Calc_TH_FD2_2
Calc_TH_FD1:
;1
       MOV   R30,R19
       RCALL SymToBufDisp
;2
Calc_TH_FD1_2:
       MOV   R30,R20
       RCALL SymToBufDisp
;3
       MOV   R30,R21
       RCALL SymOrPointToBufDisp
;4
       MOV   R30,R22
       RCALL SymToBufDisp
;  
       RJMP  Mode_TH_Inc
;***********************************************************
;    , c  ,
;  
;***********************************************************
Calc_TH_FD2:
;1
       MOV   R30,R20
       RCALL SymToBufDisp
;2
Calc_TH_FD2_2:
       MOV   R30,R21
       RCALL SymOrPointToBufDisp
;3
       MOV   R30,R22
       RCALL SymToBufDisp
;4
       RJMP  Calc_TH_FP_4
;***********************************************************
;    ,  
;***********************************************************
Calc_TH_FDM:
       ST    Y+,R23
       TST   R20
       BRNE  Calc_TH_FD1_2
       RJMP  Calc_TH_FD2_2
;***********************************************************
SymToBufDisp:
       RCALL LPM_Number
       ST    Y+,R0
       RET
;***********************************************************
SymOrPointToBufDisp:
       RCALL LPM_Number
;  
       MOV   R30,R0
       SBR   R30,1<<7
       ST    Y+,R30
       RET
;***********************************************************
;   BCD  BIN
;***********************************************************
BCD2BIN:
       MOV   R31,R30
       ANDI  R30,$0F
       CPI   R30,10
       BRCC  BCD2BIN_Error
       SWAP  R31
       ANDI  R31,$0F
       CPI   R31,10
       BRCC  BCD2BIN_Error
       SWAP  R31
       LSR   R31
       ADD   R30,R31
       LSR   R31
       LSR   R31
       ADD   R30,R31
       CLC
       RET
BCD2BIN_Error:
       SEC
       RET
;***********************************************************
;   BIN  BCD
;***********************************************************
BIN2BCD:
       PUSH  R0
       PUSH  R1
       LDI   R31,7
       MOV   R1,R31
       LDI   R31,10
       CLR   R0
BIN2BCD_Go:
       ROL   R30
       ROL   R0
       CP    R0,R31
       BRCS  BIN2BCD_Skip_Sub
       SUB   R0,R31
BIN2BCD_Skip_Sub:
       DEC   R1
       BRPL  BIN2BCD_Go
       ROL   R30
       COM   R30
       ANDI  R30,$0F
       SWAP  R30
       OR    R30,R0
       POP   R1
       POP   R0
       RET
;**********************************************************
;**********************************************************
R19_R18_Mul_10:
       CLR   R21
       LDI   R20,10
;***********************************************************
;  2-  
;***********************************************************
;R19_R18_Mul_R21_R20:
       CLR   R0
       CLR   R1
R19_R18_Mul_R21_R20_Go:
       CPI   R18,$00
       CPC   R18,R19
       BREQ  R19_R18_Mul_R21_R20_End
       LSR   R19
       ROR   R18
       BRCC  R19_R18_Mul_R21_R20_Skip_Add
       ADD   R0,R20
       ADC   R1,R21
R19_R18_Mul_R21_R20_Skip_Add:
       LSL   R20
       ROL   R21
       RJMP  R19_R18_Mul_R21_R20_Go
R19_R18_Mul_R21_R20_End:
       MOV   R18,R0
       MOV   R19,R1
       RET
;**********************************************************
;**********************************************************
R19_R18_Div_16:
       CLR   R21
       LDI   R20,16
       RJMP  R19_R18_Div_R21_R20
R19_R18_Div_10:
       CLR   R21
       LDI   R20,10
;***********************************************************
;  2-  
;***********************************************************
R19_R18_Div_R21_R20:
       CLR   R0
       CLR   R1
       LDI   R30,$0F
R19_R18_Div_R21_R20_Go:
       ROL   R18
       ROL   R19
       ROL   R0
       ROL   R1
       CP    R0,R20
       CPC   R1,R21
       BRCS  R19_R18_Div_R21_R20_Skip_Sub
       SUB   R0,R20
       SBC   R1,R21
R19_R18_Div_R21_R20_Skip_Sub:
       DEC   R30
       BRPL  R19_R18_Div_R21_R20_Go
       ROL   R18
       ROL   R19
       MOV   R20,R0
       MOV   R21,R1
       COM   R18
       COM   R19
       RET
;***********************************************************
; 
;***********************************************************
Dop_R19_R18:
       CLR   R30
       SUB   R30,R18
       MOV   R18,R30
       CLR   R30
       SBC   R30,R19
       MOV   R19,R30
       RET
;***********************************************************
;***********************************************************
Inc_R19_R18:
       INC   R18
       BRNE  Inc_R19_R18_End
       INC   R19
Inc_R19_R18_End:
       RET
;***********************************************************
; TEMPERATURE = (MSB LSB)/16
;  : R19-R18
;  : Code_TH
;***********************************************************
CalcThDS18B20:
       CLR   R22
;    ?
       SBRC  RegFlags2,F_TH_POINT
;     
       SBR   R22,1<<6
;  
       SBRS  R19,7
       RJMP  CalcThDS18B20_Plus
;    
       SBR   R22,1<<7
       RCALL Dop_R19_R18
CalcThDS18B20_Plus:
;    *10
       SBRC  R22,6
       RCALL R19_R18_Mul_10
       RCALL R19_R18_Div_16
; 
       SBRC  R20,3
       RCALL Inc_R19_R18
;   BD
       LDI   XL,Code_TH+5
       LDI   R31,4
CalcThDS18B20_Do:
       MOV   R20,R19
       OR    R20,R18
       BREQ  CalcThDS18B20_Z
       RCALL R19_R18_Div_10
CalcThDS18B20_Z:
       ST    -X,R20
       DEC   R31
       BRNE  CalcThDS18B20_Do
       ST    -X,R22
;      
       CBR   RegFlags2,1<<F_TH_POINT
       RET
;***********************************************************
;
;***********************************************************
Key_Work:
       TST   BufKey
       BREQ  Key_Work_End_No_Refresh
       MOV   R30,BufKey
       CLR   BufKey
       CPI   R30,Button_Enter
       BRNE  Key_Work_Inc_Dec_Reg
;  Enter
       SBRC  RegFlags1,F_Repeat_Press_Key
       RJMP  Key_Work_Enter
       SBRS  RegFlags2,F_SET_WATH
       RJMP  Key_Work_End
;  Enter    
       LDI   R30,1<<F_SET_HOURS
       EOR   RegFlags2,R30
Key_Work_End:
; 
       SBR   RegFlags1,1<<F_SHOW
Key_Work_End_No_Refresh:
       RET
;***********************************************************
;    
;***********************************************************
Key_Work_Enter:
       SBRC  RegFlags2,F_SET_WATH
       RJMP  Key_Work_Exit
;      
;  
       RCALL Read_RTC
;  
       CBR   RegFlags2,1<<F_SET_HOURS
;    
       SBR   RegFlags2,1<<F_SET_WATH
       RJMP  Key_Work_End
Key_Work_Exit:
;    
;   
       SBRC  RegFlags2,F_WRITE_WATH
;  
       RCALL Write_RTC_Setup
;  
       RCALL Read_RTC
Key_Work_Exit_Skip_Write:
;    
       CBR   RegFlags2,1<<F_SET_WATH
       RJMP  Key_Work_End
;***********************************************************
; / 
;***********************************************************
Key_Work_Inc_Dec_Reg:
       SBRS  RegFlags2,F_SET_WATH
       RJMP  Set_Show_Par
;     (10    )
       CPI   R30,Button_Escape
       BREQ  Key_Work_Exit_Skip_Write
; Button_Up
       CPI   R30,Button_Up
       BRNE  Inc_Dec_Reg_No_Button_Up
       LDI   R30,1
       RJMP  Inc_Dec_Reg_Par
Inc_Dec_Reg_No_Button_Up:
; Button_Down
       CPI   R30,Button_Down
       BRNE  Inc_Dec_Reg_No_Button_Down
       LDI   R30,$FF
       RJMP  Inc_Dec_Reg_Par
Inc_Dec_Reg_No_Button_Down:
       RET
Inc_Dec_Reg_Par:
;  
       RCALL Inc_Dec_Reg
;    
       SBR   RegFlags2,1<<F_WRITE_WATH
;      
       CLR   mSecondsBlink
       CBR   RegFlags1,1<<F_Blink
       RJMP  Key_Work_End
;***********************************************************
;***********************************************************
Set_Show_Par:
; Button_Up
       CPI   R30,Button_Up
       BREQ  Set_Show_Par_TH
; Button_Down
       CPI   R30,Button_Down
       BREQ  Set_Show_Par_Time
       RET
Set_Show_Par_TH:
;    
       CLR   WaitShow
       CBR   RegFlags2,1<<F_SHOW_WATH
;      
       SBR   RegFlags2,1<<F_TH_POINT
;    
       CLR   Seconds
       SBR   RegFlags1,1<<F_Read_TH
       RJMP  Key_Work_End
Set_Show_Par_Time:
;    
       CLR   WaitShow
       SBR   RegFlags2,1<<F_SHOW_WATH
       RJMP  Key_Work_End
;***********************************************************
; /  
;***********************************************************
Inc_Dec_Reg:
;    
       SBRS  RegFlags2,F_SET_HOURS
       RJMP  Inc_Dec_Reg_Minites_Set
       LDI   XL,SHours
       LD    R31,X
       ADD   R31,R30
       CPI   R31,24
       BRCS  Inc_Dec_Reg_HR
       CLR   R31
       SBRC  R30,7
       LDI   R31,23
Inc_Dec_Reg_HR:
       ST    X,R31
       RET
Inc_Dec_Reg_Minites_Set:
       LDI   XL,SMinutes
       LD    R31,X
       ADD   R31,R30
       CPI   R31,60
       BRCS  Inc_Dec_Reg_MR
       CLR   R31
       SBRC  R30,7
       LDI   R31,59
Inc_Dec_Reg_MR:
       ST    X,R31
       RET
;***********************************************************
;***********************************************************
Init_Mem_TH:
       LDI   XL,TL0
       LDI   YL,TL0+1*2
       LDI   R31,4*2
Init_Mem_TH_Loop:
       LD    R30,X+
       ST    Y+,R30
       DEC   R31
       BRNE  Init_Mem_TH_Loop
       RET
;***********************************************************
;   
;***********************************************************
Mem_Up_Down_TH:
;    
       LDI   XL,TL0+4*2
       LDI   YL,TL0+5*2
       LDI   R31,4*2
Mem_Up_Down_TH_Push:
       LD    R30,-X
       ST    -Y,R30
       DEC   R31
       BRNE  Mem_Up_Down_TH_Push
;    
       LDI   XL,LSB
       LDI   YL,TL0
       LD    R30,X+
       ST    Y+,R30
       LD    R30,X
       ST    Y,R30
;       
       SBRS  RegFlags1,F_Enable_Show_TH
       RCALL Init_Mem_TH
;     15 
       LDI   YL,TL0
       LD    R18,Y
       LDD   R19,Y+1
       LDD   R20,Y+4*2
       LDD   R21,Y+4*2+1
       SUB   R18,R20
       SBC   R19,R21
;     
       CBR   RegFlags1,(1<<F_TH_Up) | (1<<F_TH_Down)
       SBRC  R19,7
       RJMP  Mem_Up_Down_TH_COM
       RCALL Compare_TH_Min_Value
       BRCC  Mem_Up_Down_TH_Ok
;    
       SBR   RegFlags1,1<<F_TH_Up
Mem_Up_Down_TH_Ok:
       RET
Mem_Up_Down_TH_COM:
       RCALL Dop_R19_R18
       RCALL Compare_TH_Min_Value
       BRCC  Mem_Up_Down_TH_Ok
;    
       SBR   RegFlags1,1<<F_TH_Down
       RET
;***********************************************************
;***********************************************************
Compare_TH_Min_Value:
       TST   R19
       BRNE  Compare_TH_Min_Value_Set
       CPI   R18,Min_Value_TH_Up_Down
       BRCC  Compare_TH_Min_Value_Set
       CLC
       RET
Compare_TH_Min_Value_Set:
       SEC
       RET
